package unionpay

import (
	"crypto"
	"crypto/rand"
	"crypto/rsa"
	"crypto/x509"
	"encoding/pem"
	"errors"
	"fmt"
	"io/ioutil"
)

// RSAEncrypt RSA 加密
func RSAEncrypt(publicKey *rsa.PublicKey, src []byte) (data []byte, err error) {
	data, err = rsa.EncryptPKCS1v15(rand.Reader, publicKey, src)
	return
}

// RSADecrypt RSA 解密
func RSADecrypt(priveKey *rsa.PrivateKey, ciphertext []byte) (data []byte, err error) {
	data, err = rsa.DecryptPKCS1v15(rand.Reader, priveKey, ciphertext)

	return
}

// RSASign256 RSA sha256 签名
func RSASign256(priveKey *rsa.PrivateKey, src []byte) (data []byte, err error) {
	h := crypto.Hash.New(crypto.SHA256)
	_, err = h.Write(src)
	if err != nil {
		return
	}
	hashed := h.Sum(nil)
	data, err = rsa.SignPKCS1v15(rand.Reader, priveKey, crypto.SHA256, hashed)
	return
}

// GetCertificate 获取证书信息
func GetCertificate(filename string) (cert *x509.Certificate, data []byte, err error) {
	data, err = ioutil.ReadFile(filename)
	if err != nil {
		return
	}

	block, _ := pem.Decode(data)
	if block == nil {
		err = errors.New(`pem Decode error`)
		return
	}

	cert, err = x509.ParseCertificate(block.Bytes)
	return
}

// GetPrivateKey 获取私钥信息
func GetPrivateKey(filename string) (key *rsa.PrivateKey, err error) {
	var (
		bs      []byte
		private interface{}
	)
	bs, err = ioutil.ReadFile(filename)
	if err != nil {
		fmt.Println(err)
		return
	}
	block, _ := pem.Decode(bs)
	if block == nil {
		err = errors.New(`pem Decode error`)
		return
	}
	private, err = x509.ParsePKCS8PrivateKey(block.Bytes)
	if err != nil {
		return
	}
	key = private.(*rsa.PrivateKey)
	return
}
